#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Operators {
  /// <summary>
  /// An operator which increments an integer variable.
  /// </summary>
  [Item("AllSubScopesIntReducer", "An operator to reduce int values of sub scopes")]
  [StorableClass]
  public sealed class AllSubScopesIntReducer : SingleSuccessorOperator {

    public ScopeTreeLookupParameter<IntValue> ParameterToReduce {
      get { return (ScopeTreeLookupParameter<IntValue>)Parameters["ParameterToReduce"]; }
    }

    public ValueLookupParameter<StringValue> TargetParameterName {
      get { return (ValueLookupParameter<StringValue>)Parameters["TargetParameterName"]; }
    }

    private LookupParameter<IntValue> TargetParameter {
      get { return (LookupParameter<IntValue>)Parameters["TargetParameter"]; }
    }

    [StorableConstructor]
    private AllSubScopesIntReducer(bool deserializing) : base(deserializing) { }
    private AllSubScopesIntReducer(AllSubScopesIntReducer original, Cloner cloner)
      : base(original, cloner) {
    }
    public AllSubScopesIntReducer()
      : base() {
      Parameters.Add(new ScopeTreeLookupParameter<IntValue>("ParameterToReduce", "The actual parameter defined by TargetParameterName"));
      Parameters.Add(new ValueLookupParameter<StringValue>("TargetParameterName", "Parameter name to add the value to", new StringValue(string.Empty)));
      Parameters.Add(new LookupParameter<IntValue>("TargetParameter", "The actual parameter defined by TargetParameterName"));
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new AllSubScopesIntReducer(this, cloner);
    }

    public override IOperation Apply() {
      var intValues = ParameterToReduce.ActualValue;

      if (intValues.Count() > 0) {
        int sum = intValues.Sum(x => x.Value);

        TargetParameter.ActualName = TargetParameterName.ActualValue.Value;
        if (TargetParameter.ActualValue == null) TargetParameter.ActualValue = new IntValue();
        TargetParameter.ActualValue.Value += sum;
      }
      return base.Apply();
    }
  }
}
